#include "Connection.h"
#include <sstream>

int Connection::initSocket(){
  /* get an internet domain socket */
  if ((sd_listen = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
    perror("socket");
    return 1;
  }
 
  /* lose the pesky "address already in use" error message */
  int yes = 1;
  if(setsockopt(sd_listen, SOL_SOCKET, SO_REUSEADDR, &yes,
		 sizeof(int)) == -1) {
    perror("setsockopt");
    return 1;
  }
  
  
  /* complete the socket structure */
  memset(&sin, 0, sizeof(sin));
  sin.sin_family = AF_INET;
  sin.sin_addr.s_addr = INADDR_ANY;
  sin.sin_port = htons(PORT);
  
  /* bind the socket to the port number */
  if(bind(sd_listen, (struct sockaddr *) &sin, sizeof(sin)) == -1){
    perror("bind");
    return 1;
  }
  
  /* show that we are willing to listen */
  if(listen(sd_listen, 5) == -1){
    perror("listen");
    return 1;
  }
  return 0;
}

void Connection::closeSockets(bool closel){
  if(closel){
    close(sd_listen);
  }
  close(sd_current);
}

int Connection::wait4kiss(){
  socklen_t addrlen;
  
  /* wait for a client to talk to us */
  if((sd_current = accept(sd_listen, (struct sockaddr *)&pin, &addrlen)) == -1) {
    perror("accept");
    return 1;
  }
  
  return 0;
}

int Connection::handleRequest(){
  string sRequest;
  int rev = 0;
  bool bNotDone = true;
  while(bNotDone){
    if((rev = getData(sRequest)) == 0){    
      kRequest kReq = getRequestType(sRequest);
      switch(kReq){
      case rLIST:
	rev = handleList(sRequest);
	break;
      case rACTION1:
	rev = handleAction1(sRequest);
	break;
      case rACTION2:
	rev = handleAction1(sRequest);
	break;
      case rGET:
	rev = handleGet(sRequest, -1);
	break;
      case rCLOSED:
	bNotDone = false;
	break;
      default:
	cout << "Unknown KiSS Command\n";
	cout << sRequest << endl;
      }
    }
    else{
      bNotDone = false;
    }
  }

  close(sd_current);
  return rev;
}

int Connection::handleList(string &listRequest){
  lRequest lReq;
  if((lReq = getListType(listRequest)) == lERROR){
    cout << "Unknown List Command\n";
    return 0;
  }
  
  string listData;
  
  switch(lReq){
  case lAUDIO:
    listData = kissData.getAudioList(listRequest);
    break;
  case lVIDEO:
    listData = kissData.getVideoList(listRequest);
    break;
  case lPICTURE:
    listData = kissData.getPictureList(listRequest);
    break;
  }

  if(listData.length() != 0){
    if(sendData(listData) == -1){
      return 1;
    }
    else{
      return 0;
    }
  }

  return 0;
}

int Connection::handleAction1(string &action1Request){
  int fdfile, fstart, fend;
  string filename, sActionOk;
  
  fstart = action1Request.find("/");
  fend = action1Request.rfind("|");
  
  filename = action1Request.substr(fstart, fend - fstart);

  if((fdfile = open(filename.c_str(), O_RDONLY)) == -1){
    perror("open");
    cout << "File Not Found: " << filename << endl;
    sActionOk = "404";
    if(sendData(sActionOk)){
      return 1;
    }
    return 0;
  }
  else{
    sActionOk = "200";
    if(sendData(sActionOk)){
      return 1;
    }
  }
  
  string sCommand;
  int rev = 0;
  bool bNotDone = true;
  
  while(bNotDone){
    if(getData(sCommand)){
      bNotDone = false;
      rev = 1;
    }
    else{
      kRequest kReq = getRequestType(sCommand);
      switch(kReq){
      case rSIZE:
	if(handleSize(sCommand)){
	  bNotDone = false;
	  rev = 1;
	}
	break;
      case rGET:
	if(handleGet(sCommand, fdfile)){
	  bNotDone = false;
	  rev = 1;
	}
	break;
      case rCLOSED:
	bNotDone = false;
	break;
      default:
	cout << "KiSS On Drugs...\n";
      }
    }
  }

  close(fdfile);
  return rev;
}

int Connection::handleGet(string &getRequest, int fdfile){
  string filename;
  int offset, chunk;
  bool fileOpened = false;
  if(parseGet(getRequest, filename, offset, chunk)){
    cout << "Not A Get Command!\n";
    return 0;
  }
  
  int lfdfile;
  if(fdfile == -1){
    if((lfdfile = open(filename.c_str(), O_RDONLY)) == -1){
      perror("open");
      return 0;
    }
    else{
      fileOpened = true;
    }
  }
  else{
    lfdfile = fdfile;
  }
  
  if(chunk == 0){
    if((chunk = getFileSize(filename)) == -1){
      close(lfdfile);
      return 0;
    }
  }
  
  if(sendfile(sd_current, lfdfile, (off_t*)&offset, chunk) == -1){
    perror("sendfile");
    close(lfdfile);
    return 1;
  }
  
  if(fileOpened){
    close(lfdfile);
  }

  return 0;
}

int Connection::handleSize(string &sizeRequest){
  string filename = sizeRequest.substr(5,sizeRequest.find("|") - 5);
  
  int ifsize;
  if((ifsize = getFileSize(filename)) == -1){
    cout << "File Not Found: " << filename << endl;
    return 0;
  }
  
  stringstream convert;
  convert << ifsize;
  convert << endl;
  string fsize;
  convert >> fsize;
  
  char kfsize[] = "000000000000000";
  for(int i = 0; i < fsize.length(); i++){
    kfsize[14 - i] = fsize[fsize.length() - i -1];
  }

  string filesize = kfsize;
  return sendData(filesize);
}

int Connection::getData(string &data){
  char buffer[2048];
  int rlen;

  /* get a message from the client */
  if((rlen = recv(sd_current, buffer, sizeof(buffer), 0)) == -1){
    perror("recv");
    data = "ERROR";
    return 1;
  }
  
  if(rlen == 0){
    cout << "KiSS Closed Connection\n";
    data = "NODATA";
    return 0;
  }
  
  buffer[rlen] = 0x0;
  data = buffer;
  return 0;
}

int Connection::sendData(string &data){  
  if(send(sd_current, data.c_str(), strlen(data.c_str()), 0) == -1){
    perror("send");
    return 1;
  }

  return 0;
}

kRequest Connection::getRequestType(string &sRequest){
  if(sRequest.find("LIST") == 0)
    return rLIST;
  if(sRequest.find("ACTION 1") == 0)
    return rACTION1;
  if(sRequest.find("ACTION 2") == 0)
    return rACTION2;
  if(sRequest.find("SIZE") == 0)
    return rSIZE;
  if(sRequest.find("GET") == 0)
    return rGET;
  if(sRequest == "NODATA")
    return rCLOSED;
  
  return rERROR;
     
}

lRequest Connection::getListType(string &listRequest){
  if(listRequest.find("AUDIO") == 5)
    return lAUDIO;
  if(listRequest.find("VIDEO") == 5)
    return lVIDEO;
  if(listRequest.find("PICTURE") == 5)
    return lPICTURE;
  
  return lERROR;
}

int Connection::parseGet(string &getRequest,string &sFileName, int &iOffSet, int &iChunk){
  int fstart, fend;
  string sFileOps;
  stringstream convert;
  
  fstart = getRequest.find("/");  
  fend = getRequest.find("|");
  
  sFileName = getRequest.substr(fstart, fend - fstart);  
  sFileOps = getRequest.substr(fend + 1, getRequest.length() - fend);

  convert << sFileOps << endl;
  convert >> iOffSet >> iChunk;

  return 0;
}

int Connection::getFileSize(string filename){
  struct stat filesize;
  
  if(stat(filename.c_str(), &filesize) == -1){
    perror("stat");
    return -1;
  }

  return filesize.st_size;
}
