#include "ServerData.h"
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <cctype>
#include <ctype.h>
#include <algorithm>
#include <vector>

using namespace std;

ServerData::ServerData(){
}

string ServerData::getAudioList(string &folderRequest){
  string sFolder = sAudioPath;
  string rFolder = getFolderRequested(folderRequest, sFolder);
  return listFolder(rFolder);
}

string ServerData::getVideoList(string &folderRequest){
  string sFolder = sVideoPath;
  string rFolder = getFolderRequested(folderRequest, sFolder);
  return listFolder(rFolder);
}

string ServerData::getPictureList(string &folderRequest){
  string sFolder = sPicturePath;
  string rFolder = getFolderRequested(folderRequest, sFolder);
  return listFolder(rFolder);
}

string ServerData::getFolderRequested(string &folderRequest, string &prefix){
  string folder;
  int firstpipe;
  int lastpipe;

  firstpipe = folderRequest.find("|");
  lastpipe = folderRequest.rfind("|");

  if((firstpipe + 1) == lastpipe){
    return prefix;
  }
  
  folder = prefix;
  folder += "/";
  folder += folderRequest.substr(firstpipe + 1, lastpipe - firstpipe - 1);
  return folder;
}

string ServerData::listFolder(string folderPath){
  string fData;
  vector<string> vDirs;
  vector<string> vFiles;

  DIR *theDir = NULL;
  theDir = opendir(folderPath.c_str());
 
  if(theDir == NULL){
    cout << "No such dir: " << folderPath << "\n";
    return fData;
  }

  dirent *theEnts;
  struct stat results;
  while((theEnts = readdir(theDir)) != NULL){
    string fname = theEnts->d_name;   
    string pfname = folderPath + "/" + fname;
    if((fname != ".") && (fname != "..")){      
      if(stat(pfname.c_str(), &results) == 0){
	fData = fname;
	fData += "| ";
	if(S_ISDIR(results.st_mode)){
	  fData += fname;
	  fData += "|1|\n";
	  vDirs.push_back(fData);
	}
	else{
	  fData += pfname;
	  fData += "|0|\n";
	  vFiles.push_back(fData);
	}	
      }
      else{
	cout << "No such file: " << fname << "\n";    
      }
    }
  }
  
  closedir(theDir);
  
  sort(vDirs.begin(), vDirs.end());
  sort(vFiles.begin(), vFiles.end());
  
  fData = "";
  
  for(int i = 0; i < vDirs.size(); i++){
    fData += vDirs[i];
  }

  for(int i = 0; i < vFiles.size(); i++){
    fData += vFiles[i];
  }
  

  fData += "EOL\n";
  return fData;
}

int ServerData::parseConfigFile(){
  
  string sHomeDir;
  char * hname;
  char buffer[512];

  if((hname = getenv("HOME")) == NULL){
    cout << "HOME not set!\n";
    return 1;
  }
  
  sHomeDir = hname;
  string cFilePath = sHomeDir + "/" + ".kiss4lin.conf";
  ifstream cFile(cFilePath.c_str());

  if(!cFile.is_open()){
    cout << "Config file not found!\n";
    return 1;
  }

  while(cFile.getline(buffer, 512)){
    string sLine = buffer;
    string sOption, sValue;
    bool bValidOption = false;
    removeSpaces(sLine);
    if(sLine.length()){
      if(sLine[0] != '#'){	
	if(getStringPair(sLine, sOption, sValue)){
	  cout << "Error in config file!\n";
	  return 1;
	}
	
	if(sOption == "audiopath"){
	  sAudioPath = sValue;
	  bValidOption = true;
	}

	if(sOption == "videopath"){
	  sVideoPath = sValue;
	  bValidOption = true;
	}

	if(sOption == "picturepath"){
	  sPicturePath = sValue;
	  bValidOption = true;
	}

	if(!bValidOption){
	  cout << "Unknown option: " << sOption << endl;
	  return 1;	  
	}
      }
    }
  }
  
  return 0;
}

int ServerData::getStringPair(string sLine, string &sOption, string &sValue){
  int pEquals;
  if((pEquals = sLine.find("=")) == string::npos){
    return 1;
  }

  sOption = sLine.substr(0, pEquals - 1);
  removeSpaces(sOption);
  
  sValue = sLine.substr(pEquals + 1, sLine.length() - pEquals);
  removeSpaces(sValue);
    
  return 0;
}

void ServerData::removeSpaces(string &sSubject){
  string s,ss;
  int spos = 0;

  while(isspace(sSubject[spos])){
    spos++;
  }
  s = sSubject.substr(spos);
  
  spos = s.length() - 1;

  while(isspace(s[spos])){
    spos--;
  }

  ss = s.substr(0, spos + 1);
  
  sSubject = ss;  
}
